;***************  MPU-401 Command file  01/20/88  *****************************

		PAGE	,132
		TITLE	'DOS-MPU.ASM Version 2.0'

code segment public 'code'
	assume cs:code,ds:code,es:code
		org 100h

begin:		jmp main

DATAPORT	equ 330h
STATPORT	equ 331h
COMPORT		equ 331h

DSR		equ 80h
DRR		equ 40h

NO_CLOCKS	equ 032h
RESET		equ 0FFh
SYSEX_THRU	equ 037h

programmer	db '(C) Copyright 1989 Larry Leenerts',1Ah
ok_ack		db ' ACK recieved, hex value is ->$'
no_ack		db ' No ACK recieved, hex return value ->$'
lines		db '-------------$'
reset_mpu	db ' SUCCESSFUL MPU-401 SYSTEM RESET$'
no_reset	db ' NO MPU-401 SYSTEM RESET...try again.$'
clocks		db ' MIDI CLOCK COMMANDS <F8-FC> DISABLED$'
sys_x		db ' SYSTEM EXCLUSIVE THRU ENABLED$'
usage		db 13,10,'Usage:  DOS-MPU [H][R][X][C]',13,10,13,10
		db 'Where ->',09
		db 'H = HELP MENU',13,10,09,09
		db 'R = MPU-401 SYSTEM RESET',13,10,09,09
		db 'X = SYSTEM EXCLUSIVE THRU',13,10,09,09
		db 'C = REAL TIME CLOCK COMMANDS <F8-FC> DISABLED',13,10,13,10
		db 'These commands may be mixed together in any order, '
		db 'but remember the ',39,'R',39,13,10
		db 'command RESETS the MPU-401, disabling any previous '
		db 'command line options!',13,10,'$'
wait_msg	db 13,10,'PLEASE WAIT$'
cr_lf		db 13,10,'$'
dot_type	db '.'
dot_count	db 0
DIGIT_BUF	db 3 dup(20h)
		db '$'

;******************************************************************************
;
;	MAIN		Main procedure
;
;******************************************************************************

main	proc 	near
	mov	si,81h			;command line starts here
	call	next_parm		;look for spaces,or cr and update SI
	jnc	input    		;if carry flag clear, start input parse
	mov	dx,offset usage
	call	print_string
	jmp	short exit
get_specs:
	mov	dx,offset cr_lf
	call	print_string
	inc	si
	call	next_parm		;look for more
	jc	exit    		;if carry flag set, exit program
input:
	and	byte ptr [si],0DFh	;uppercase input char
	cmp	byte ptr [si],'H'
	jne	no_clock
help:
	mov	dx,offset usage
	call	print_string
	jmp	get_specs
no_clock:
	cmp	byte ptr [si],'C'
	jne	mpu_reset
	mov	ax,NO_CLOCKS
	call	mpu_command
	call	check_al
	jc	exit
	mov	dx,offset clocks
	call	print_line
	jmp	get_specs
mpu_reset:
	cmp	byte ptr [si],'R'
	jne	sysex
do_reset:
	mov	ax,RESET
	call	mpu_command
	jnc	chk_reset
	mov	ax,RESET
	call	mpu_command
chk_reset:
	call	check_al
	jc	no_mpu
	mov	dx,offset reset_mpu
	call	print_line
	jmp	get_specs
no_mpu:
	mov	dx,offset no_reset
	call	print_line
	jmp	short exit
sysex:
	cmp	byte ptr [si],'X'
	jne	get_specs
	mov	ax,SYSEX_THRU
	call	mpu_command
	call	check_al
	jc	exit
	mov	dx,offset sys_x
	call	print_line
	jmp	get_specs
exit:	int	20h
main	endp

;******************************************************************************
;
;	NEXT_PARM	Get the next parameter on command line
;
;******************************************************************************

next_parm proc near
	cmp 	byte ptr [si],13	;is this a carriage return?
	je  	eol			;if so set carry flag and exit
	cmp  	byte ptr [si],32	;is this a space?
	jne  	no_sp			;if not, clear carry flag and exit
	inc  	si			;increment string pointer
	jmp  	next_parm		;and go back for more
no_sp:	clc				;clear carry flag
	ret				;and return to caller
eol:	stc				;set carry flag
	ret				;and return to caller
next_parm endp

;******************************************************************************
;
;	MPU_COMMAND	Assembly subroutine for accessing the MPU-401 ports
;
;******************************************************************************

mpu_command	proc	near
		push	ax		;save command byte for later
		mov	al,'>'
		mov	byte ptr [dot_type],al
		mov 	dx,331h		;com_stat_port
		mov	cx,4096
check_drr:
		in 	al,dx		;test DRR (bit 6)
		and 	al,40h
		jz	ok_6
		cmp 	cx,3841
		jnb 	loop_again
		call	print_dots
		mov 	bx,50
delay:
		dec 	bx
		jnz 	delay
loop_again:
		loop	check_drr
		pop	ax
		jmp	short set_carry
ok_6:
		mov	al,'.'
		mov	byte ptr [dot_type],al
		pop	ax		;get command byte in AX
		clc			;clear carry flag
		cli			;disable interrupts
		out 	dx,al		;send command to com_stat_port
		mov	cx,4096
check_dsr:	in 	al,dx		;test DSR (bit 7)
		and 	al,80h
		jz	ok_7
		cmp 	cx,3841
		jnb 	loop_next
		call	print_dots
		mov 	bx,50
ndelay:
		dec 	bx
		jnz 	ndelay
loop_next:
		loop	check_dsr
		jmp	short set_carry
ok_7:
		mov 	dx,330h		;dataport
		in 	al,dx		;read dataport 
		cmp 	al,0FEh		;does it contain an ACK (FEh)?
		je 	exit_out	;if so, return to caller with no carry
set_carry:
		push	ax
		mov	dx,offset cr_lf
		call	print_string
		pop	ax
		stc			;set carry flag
exit_out:	sti			;restore interrupts
		ret			;return from sub_routine
mpu_command	endp

;*****************************************************************************
;
;	CHECK_AL	checks the byte value in AX
;
;*****************************************************************************

check_al proc	near
	pushf
	push	ax
	mov	dx,offset cr_lf
	call	print_string
	pop	ax
	mov	dx,offset ok_ack
	cmp 	al,0FEh			;does AL contain an ACK?
	je	print_ack
	mov	dx,offset no_ack
print_ack:
	push	ax
	push	dx
	mov	dx,offset lines
	call	print_string
	pop	dx
	call	print_string
	pop	ax
	xor	ah,ah
	call	hex_out
	mov	dx,offset cr_lf
	call	print_string
	popf
	ret		
check_al	endp

;******************************************************************************
;
;	HEX_OUT		output value in AX in HEX to console
;
;******************************************************************************

hex_out	proc	near
	mov	di,offset digit_buf	; string buffer address
	push	di			; save for later
	add	di,2			; point to last digit
	std				; set for auto decrement
	call	hexdig_out0		; store two least significant digits
	pop	dx			; pt DX to beginning of string
	call	print_string
	ret

hexdig_out0:
	push	ax			; save AL for other half of digit
	and	al,0Fh			; strip off high digit
	call	hexdig_out1		; store lower digit
	pop	ax			; get AL back for high digit
	shr	al,1			; shift into lower digit
	shr	al,1			;
	shr	al,1			;
	shr	al,1			;
					; then store lower digit
hexdig_out1:
	or	al,30h			; convert 0-9 to ascii
	cmp	al,3Ah			;
	jb	hexdig_out2		;
	add	al,7			; convert 10-15 to A-F

hexdig_out2:
	stosb				; store digit in buffer & pt to next
	ret				; most significant digit
hex_out	endp

;******************************************************************************
;
;	PRINT_DOTS	print dots to the standard output
;
;******************************************************************************

print_dots	proc	near
		push 	cx
		push 	dx

		cmp	byte ptr [dot_count],0
		jnz	go_ahead
		mov	dx,offset wait_msg
		call	print_string
		jmp	dot_new
go_ahead:
		cmp	byte ptr [dot_count],100
		jz	dot_new
		inc	dot_count
		jmp	dot_exit
dot_new:
		mov	byte ptr [dot_count],1
		mov	dl,byte ptr [dot_type]		; > DRR or '.' DSR
		mov 	ah,2				;DOS display character
		int 	21h
dot_exit:
		pop 	dx
		pop 	cx
		ret
print_dots	endp

;******************************************************************************
;
;	PRINT_STRING	print a string to the standard output
;
;******************************************************************************

print_string:	mov 	ah,9		;DOS print string function
		int 	21h		;call DOS
		ret

;******************************************************************************
;
;	PRINT_LINE	print a selected option to the standard output
;
;******************************************************************************

print_line:	mov 	ah,9
		push	dx		;save pointer for string to print
		mov	dx,offset lines
		int 	21h
		pop	dx
		int 	21h
		mov	dx,offset cr_lf
		int 	21h
		ret

code 	ends
end	begin
